package Wasm::Table;

use strict;
use warnings;
use 5.008004;

# ABSTRACT: Interface to WebAssembly Tables
our $VERSION = '0.23'; # VERSION


1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Wasm::Table - Interface to WebAssembly Tables

=head1 VERSION

version 0.23

=head1 DESCRIPTION

B<WARNING>: WebAssembly and Wasmtime are a moving target and the
interface for these modules is under active development.  Use with
caution.

L<Wasm> doesn't yet provide an interface to WebAssembly Tables,
but one day this is where its interface will be documented.

=head1 SEE ALSO

=over 4

=item L<Wasm>

=back

=head1 AUTHOR

Graham Ollis <plicease@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2020-2022 by Graham Ollis.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
